/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { channelsTreeName } from "@fluidframework/runtime-definitions/internal";
/**
 * Class to build paths for nodes in a tree with escaped special characters
 */
export class EscapedPath {
    constructor(path) {
        this.path = path;
    }
    /**
     * Creates and returns a new instance of this class.
     * @param path - Id or path part of a node
     */
    static create(path) {
        return new EscapedPath(encodeURIComponent(path));
    }
    toString() {
        return this.path;
    }
    /**
     * Creates and returns a new instance of this class for child of the current node.
     */
    createChildPath(childNodePath) {
        return new EscapedPath(`${this.path}/${encodeURIComponent(channelsTreeName)}/${childNodePath.path}`);
    }
}
//# sourceMappingURL=summarizerNodeUtils.js.map